#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QtGui/QMessageBox>
#include <QtDBus/QDBusInterface>
#include <QtDBus/QDBusReply>

#include "vollabel.h"
#include "haldev.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), connection(QDBusConnection::systemBus()),
      manager("org.freedesktop.Hal","/org/freedesktop/Hal/Manager",connection,NULL),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(&manager,SIGNAL(DeviceAdded(const QString &)),
            this, SLOT(DeviceAdded(QString)));
    connect(&manager,SIGNAL(DeviceRemoved(const QString &)),
            this, SLOT(DeviceRemoved(QString)));
    QStringList volumes = manager.FindDeviceByCapability("volume");
    foreach(QString volume, volumes)
        DeviceAdded(volume);
}

QString MainWindow::GetStringProperty(const QString & udi, const QString & property)
{
    return org::freedesktop::Hal::Device("org.freedesktop.Hal",udi,
                        connection).GetPropertyString(property).value();
}

int MainWindow::GetIntProperty(const QString & udi, const QString & property)
{
    return org::freedesktop::Hal::Device("org.freedesktop.Hal",udi,
                        connection).GetPropertyInteger(property).value();
}

void MainWindow::DeviceAdded(const QString & dev)
{
    QString category = GetStringProperty(dev,"info.category");
    if(category != "volume")
        return;
    QWidget * widget = new HalVolumeLabel(this,dev,connection);
    ui->devlistLayout->insertWidget(1,widget);
    devices.insert(dev,widget);
}

void MainWindow::DeviceRemoved(const QString & dev)
{
    // QHash::operator [] returns a null pointer for elements not in the map,
    // so it really works without checking.
    delete devices[dev];
}

MainWindow::~MainWindow()
{
    delete ui;
}
