#include "vollabel.h"
#include <QtDBus/QDBusArgument>

Q_DECLARE_METATYPE(HalPropertyChange)
Q_DECLARE_METATYPE(QList<HalPropertyChange>)

QDBusArgument operator<<(QDBusArgument &arg, const HalPropertyChange & propchg)
{
    arg.beginStructure();
    arg << propchg.PropertyName;
    arg << propchg.removed;
    arg << propchg.added;
    arg.endStructure();
    return arg;
}

QDBusArgument operator>>(const QDBusArgument &arg, HalPropertyChange & propchg)
{
    arg.beginStructure();
    arg >> propchg.PropertyName;
    arg >> propchg.removed;
    arg >> propchg.added;
    arg.endStructure();
    return arg;
}

void HalVolumeLabel::update_label()
{
    if(mountpoint.isEmpty())
        setText(description + " (not mounted)");
    else
        setText(description + " (at " + mountpoint + ")");
}

HalVolumeLabel::HalVolumeLabel(QWidget * parent, const QString & udi, QDBusConnection conn) :
        QLabel(parent),
        intf("org.freedesktop.Hal",udi,conn)
{
    qDBusRegisterMetaType<HalPropertyChange>();
    qDBusRegisterMetaType<QList<HalPropertyChange> >();
    org::freedesktop::Hal::Device storage_device("org.freedesktop.Hal",intf.GetPropertyString("block.storage_device"),conn);
    description = storage_device.GetPropertyString("storage.vendor") + " " +
                     storage_device.GetPropertyString("storage.model");
    org::freedesktop::Hal::Device origin("org.freedesktop.Hal",storage_device.GetPropertyString("storage.originating_device"),conn);
    QDBusReply<int> VendorReply = origin.GetPropertyInteger("usb.vendor_id");
    if(VendorReply.isValid())
    {
        int vid = VendorReply.value();
        int pid = origin.GetPropertyInteger("usb.product_id");
        description += QString(" (USB %1:%2)").arg(vid,0,16).arg(pid,0,16);
    }
    if(intf.GetPropertyBoolean("volume.is_mounted"))
        mountpoint = intf.GetPropertyString("volume.mount_point");
    else
        mountpoint = "";
    connect(&intf,SIGNAL(PropertyModified(int,const QList<HalPropertyChange>&)),
            this,SLOT(propChanged(int,const QList<HalPropertyChange>&)));

    update_label();
}

void HalVolumeLabel::propChanged(int, const QList<HalPropertyChange> & changes)
{
    bool need_update = false;
    foreach(HalPropertyChange chg, changes)
    {
        if(chg.PropertyName == "volume.is_mounted")
            if(!intf.GetPropertyBoolean("volume.is_mounted"))
            {
                mountpoint = "";
                need_update = true;
            }
        if(chg.PropertyName == "volume.mount_point" && !chg.removed)
        {
            mountpoint = intf.GetPropertyString("volume.mount_point");
            need_update = true;
        }
    }
    if(need_update)
        update_label();
}
